@@ -90,52 +90,59 @@ class Strategy:
     
     def backTest(self, period = 6000, detailed = True, tickerList = None):
         result = []
+        failed = []
         for ticker in tickerList or self.getTickerList():
-            df = self.processStrategy(ticker, period = period, screener = False)
-            columns = ['Signal', 'Trigger Price', 'Date', 'Sell Action']
-            data = df[columns]
+            try:
+                df = self.processStrategy(ticker, period = period, screener = False)
+                columns = ['Signal', 'Trigger Price', 'Date', 'Sell Action', 'Adj Close']
+                data = df[columns]
 
-            trades = []
-            temp = None
-            for i in range(len(data)) :
-                signal = data.loc[i, 'Signal']
-                price = data.loc[i, 'Trigger Price']
-                date = data.loc[i, 'Date']
-                action = data.loc[i, 'Sell Action']
-                if signal == 'Buy' and temp is None:
-                    temp = {
-                        'Stock':ticker
+                trades = []
+                temp = None
+                for i in range(len(data)) :
+                    signal = data.loc[i, 'Signal']
+                    price = data.loc[i, 'Trigger Price'] if data.loc[i, 'Trigger Price'] else data.loc[i, 'Adj Close']
+                    date = data.loc[i, 'Date']
+                    action = data.loc[i, 'Sell Action']
+                    if signal == 'Buy' and temp is None:
+                        temp = {
+                            'Stock':ticker
+                        }
+                        temp['Buy Price'] = price
+                        temp['Buy Date'] = date
+                    if (signal == 'Sell' or i == len(data)-1) and temp is not None:
+                        temp['Sell Price'] = price
+                        temp['Sell Date'] = date
+                        temp['Trigger'] = action
+                        temp['Days Invested'] = (date - temp['Buy Date']).days
+                        temp['Return'] = price/temp['Buy Price']
+                        trades.append(temp)
+                        temp = None
+                if detailed:
+                    result = result + trades
+                else:
+                    z = pd.DataFrame(trades)
+                    loss = z[z['Return'] < 1]
+                    win = z[z['Return'] >= 1]
+                    win_return = round(100*(win['Return'].product() - 1),2)
+                    loss_return = round(100*(loss['Return'].product() - 1),2)
+                    accuracy = round(100*len(win) / (len(loss) + len(win)),2)
+
+                    summary = {
+                        'Stock' : ticker,
+                        'Trade Count': len(z),
+                        'Return' : round((accuracy*win_return/100) + ((100-accuracy)*loss_return/100),2),
+                        'Accuracy' : accuracy,
+                        'Average Days' : round(z['Days Invested'].mean(),0),
+                        'Win Return': win_return,
+                        'Loss Return': loss_return,
+                        'RR': round(abs(win_return/loss_return),2) if loss_return != 0 else 100
                     }
-                    temp['Buy Price'] = price
-                    temp['Buy Date'] = date
-                if signal == 'Sell' and temp is not None:
-                    temp['Sell Price'] = price
-                    temp['Sell Date'] = date
-                    temp['Trigger'] = action
-                    temp['Days Invested'] = (date - temp['Buy Date']).days
-                    temp['Return'] = price/temp['Buy Price']
-                    trades.append(temp)
-                    temp = None
-            if detailed:
-                result = result + trades
-            else:
-                z = pd.DataFrame(trades)
-                loss = z[z['Return'] < 1]
-                win = z[z['Return'] >= 1]
-                win_return = round(100*(win['Return'].product() - 1),2)
-                loss_return = round(100*(loss['Return'].product() - 1),2)
-                accuracy = round(100*len(win) / (len(loss) + len(win)),2)
-                
-                summary = {
-                    'Stock' : ticker,
-                    'Trade Count': len(z),
-                    'Return' : (accuracy*win_return/100) + ((100-accuracy)*loss_return/100),
-                    'Accuracy' : accuracy,
-                    'Average Days' : round(z['Days Invested'].mean(),0),
-                    'Win Return': win_return,
-                    'Loss Return': loss_return,
-                }
-                result = result + [summary]
+                    result = result + [summary]
+            except:
+                failed = failed + [ticker]
+        
+        print(failed)
         return pd.DataFrame(result)
     
 class Strategy52WHigh(Strategy):
@@ -161,7 +168,7 @@ class Strategy52WHigh(Strategy):
         df = data.resample("W").agg(agg_dict)
         df.dropna(inplace = True)
         df['52W High'] = df['High'].rolling(window = 52).max()
-        df['Breakout'] = df['High'] > df.shift(1)['52W High']
+        df['Breakout'] = df['Adj Close'] > df.shift(1)['52W High']
         df['Signal'] = (df['Breakout']) & (df.shift(1)['Breakout'] == False)
         df['Signal'] = df['Signal'].apply(lambda x: 'Buy' if x else '-')
 
@@ -196,7 +203,7 @@ class Strategy52WHigh(Strategy):
 
         # Calculate Signals
         if screener:
-            df = df.loc[df['Signal'].where(df['Signal'] == 'Buy').last_valid_index():]
+            df = df.loc[df['Signal'].where(df['Signal'] == 'Buy').first_valid_index():]
             df.reset_index(inplace = True, drop = True)
         last_signal = None
         signal_target = None
